"""
    我的客户页面
"""

from TestDatas import global_datas as gd
import time

from PageLocators.myClient_page_locs import MyClientPageLocs as loc
from Common.basepage import Basepage


class MyClientPage(Basepage):

    # 通过url跳转到我的客户页面
    def to_myClient(self):
        self.driver.get(gd.myClient_url)

    # 通过首页的登录名来判断是否登录成功
    def is_username_exist(self):
        return self.get_element(loc.username_loc, "判断是否登录成功")

    # 判断是否跳转到 我的客户 页面
    def is_myClient_exist(self):
        return self.get_element(loc.myClientText_loc, "判断是否进入我的客户页面")

    # 填写 客户信息 新增客户
    def newClient(self):
        time.sleep(1)
        self.click_element(loc.addClient_button_loc, "我的客户页面_点击新增客户按钮")
        self.input_text(loc.customer_name_loc, "新增客户页面_输入客户姓名", gd.client_info["username"])
        self.input_text(loc.mobile_loc, "新增客户页面_输入手机号码", gd.client_info["mobile"])
        self.click_element(loc.source_select_loc, "新增客户页面_点击客户来源下拉框")
        self.click_element(loc.source_loc, "新增客户页面_选择客户来源")
        self.click_element(loc.recommend_select_loc, "新增客户页面_点击二级渠道下拉框")
        self.click_element(loc.recommend_loc, "新增客户页面_选择二级渠道")

        self.uploadFile(loc.upload_button_loc, "新增客户页面_上传文件", gd.document_url)
        self.input_text(loc.user_select_loc, "新增客户页面_搜索跟进人", gd.client_info["user"])
        self.click_element(loc.user_loc, "新增客户页面_选择跟进人")
        self.click_element(loc.customer_name_loc, "新增客户页面_点击客户姓名输入框")
        self.input_text(loc.remark_loc, "新增客户页面_输入备注", gd.client_info["remark"])
        self.click_element(loc.save_button_loc, "新增客户页面_点击保存按钮")
        time.sleep(5)

    # 判断创建客户是否成功
    def is_newClient_success(self):
        return self.get_element(loc.phone_loc, "判断新增客户是否成功")

    # 根据手机号查询操作
    # def mobile_inquire(self):
    #     mobile = gd.client_info["mobile"]
    #     self.input_text(loc.username_mobile_loc, "我的客户页面_根据新增用户的手机号进行查询", mobile)
    #     time.sleep(2)
    #     self.click_element(loc.query_button_loc, "我的客户页面_点击查询按钮")
    #     return self.get_element(loc.list_number_loc, "我的客户页面_查看列表显示几条数据")

    # 点击 列表手机号文本，跳转到客户详情页面
    def click_mobile_text(self):
        # 点击新增客户的手机号
        self.click_element(loc.phone_loc, "我的客户页面_点击手机号文本")
        time.sleep(5)
        # 点击详情按钮后，需要跳转窗口
        self.switch_window()
